from ctypes import c_uint32, c_uint, c_ulong, c_char_p
from . import clibrebound
import sys

def hash(key):
    hash_types = c_uint32, c_uint, c_ulong
    PY3 = sys.version_info[0] == 3
    if PY3:
        string_types = str,
        int_types = int,
    else:
        string_types = basestring,
        int_types = int, long,
   
    if isinstance(key, int_types):
        return c_uint32(key)
    elif isinstance(key, hash_types):
        return key
    elif isinstance(key, string_types):
        clibrebound.reb_hash.restype = c_uint32
        return c_uint32(clibrebound.reb_hash(c_char_p(key.encode('ascii'))))
    else:
        raise AttributeError("Need to pash hash an integer or string.")




import rebound
import unittest
import math 

class TestEOSn(unittest.TestCase):
    def setUp(self):
        self.sim = rebound.Simulation()
        self.sim.add(m=1)
        self.sim.add(m=1e-3,a=1,e=0.05,f=0.)
        self.sim.add(m=1e-3,a=1.6,e=0.05,f=1.)
        self.sim.move_to_com()
    
    def tearDown(self):
        self.sim = None

    def _run(self):
        tmax = 100.
        Emax = 0
        E0 = self.sim.calculate_energy()
        while self.sim.t<tmax:
            self.sim.integrate(self.sim.t+1.23456, exact_finish_time=0)
            E1 = self.sim.calculate_energy()
            Emax = max([Emax,abs((E0-E1)/E0)])
        return Emax
    
    def test_lf(self):
        self.sim.dt = 0.01*(2.*math.pi)
        self.sim.integrator = "eos"
        self.sim.ri_eos.phi0 = "lf"
        self.sim.ri_eos.phi1 = "lf"
        self.sim.ri_eos.n = 16
        self.sim.ri_eos.safe_mode = 0
        Emax = self._run()
        self.assertAlmostEqual(Emax,0.,delta=5.1e-7)
    
    def test_lf4(self):
        self.sim.dt = 0.001*(2.*math.pi)
        self.sim.integrator = "eos"
        self.sim.ri_eos.phi0 = "lf4"
        self.sim.ri_eos.phi1 = "lf4"
        self.sim.ri_eos.n = 2
        self.sim.ri_eos.safe_mode = 0
        Emax = self._run()
        self.assertAlmostEqual(Emax,0.,delta=2.6e-11)
    
    def test_lf4_2(self):
        self.sim.dt = 0.001*(2.*math.pi)
        self.sim.integrator = "eos"
        self.sim.ri_eos.phi0 = "lf4_2"
        self.sim.ri_eos.phi1 = "lf4"
        self.sim.ri_eos.n = 2
        self.sim.ri_eos.safe_mode = 0
        Emax = self._run()
        self.assertAlmostEqual(Emax,0.,delta=7.0e-12)
    
    def test_pmlf4(self):
        self.sim.dt = 0.001*(2.*math.pi)
        self.sim.integrator = "eos"
        self.sim.ri_eos.phi0 = "pmlf4"
        self.sim.ri_eos.phi1 = "lf4"
        self.sim.ri_eos.n = 2
        self.sim.ri_eos.safe_mode = 0
        Emax = self._run()
        self.assertAlmostEqual(Emax,0.,delta=7.0e-12)
    
    def test_lf6(self):
        self.sim.dt = 0.01*(2.*math.pi)
        self.sim.integrator = "eos"
        self.sim.ri_eos.phi0 = "lf6"
        self.sim.ri_eos.phi1 = "lf6"
        self.sim.ri_eos.n = 2
        self.sim.ri_eos.safe_mode = 0
        Emax = self._run()
        self.assertAlmostEqual(Emax,0.,delta=8e-14)
    
    def test_lf8(self):
        self.sim.dt = 0.03*(2.*math.pi)
        self.sim.integrator = "eos"
        self.sim.ri_eos.phi0 = "lf8"
        self.sim.ri_eos.phi1 = "lf8"
        self.sim.ri_eos.n = 1
        self.sim.ri_eos.safe_mode = 0
        Emax = self._run()
        self.assertAlmostEqual(Emax,0.,delta=2e-13)
    
    def test_lf8_6_4(self):
        self.sim.dt = 0.03*(2.*math.pi)
        self.sim.integrator = "eos"
        self.sim.ri_eos.phi0 = "lf8_6_4"
        self.sim.ri_eos.phi1 = "lf8"
        self.sim.ri_eos.n = 1
        self.sim.ri_eos.safe_mode = 0
        Emax = self._run()
        self.assertAlmostEqual(Emax,0.,delta=1.4e-13)
    
    def test_plf7_6_4(self):
        self.sim.dt = 0.01*(2.*math.pi)
        self.sim.integrator = "eos"
        self.sim.ri_eos.phi0 = "plf7_6_4"
        self.sim.ri_eos.phi1 = "lf8"
        self.sim.ri_eos.n = 1
        self.sim.ri_eos.safe_mode = 0
        Emax = self._run()
        self.assertAlmostEqual(Emax,0.,delta=1.5e-13)
    
    
if __name__ == "__main__":
    unittest.main()



import rebound
import unittest
import math
import rebound.data
import warnings
    
    
class TestIntegratorWHFast(unittest.TestCase):
    # WHDS
    def test_whfastwhds_outersolarsystem(self):
        sim = rebound.Simulation()
        rebound.data.add_outer_solar_system(sim)
        sim.integrator = "whfast"
        sim.ri_whfast.coordinates = "whds"
        sim.dt = 0.005*12.*2.*3.1415   # ~ 1/200 of a jupiter year
        e0 = sim.calculate_energy()
        sim.integrate(1000.*2.*3.1415)
        e1 = sim.calculate_energy()
        self.assertLess(math.fabs((e0-e1)/e1),2.4e-8)

    # Jacobi
    def test_whfast_outersolarsystem(self):
        sim = rebound.Simulation()
        rebound.data.add_outer_solar_system(sim)
        sim.integrator = "whfast"
        sim.ri_whfast.coordinates = "jacobi"
        sim.dt = 0.005*12.*2.*3.1415   # ~ 1/200 of a jupiter year
        e0 = sim.calculate_energy()
        sim.integrate(1000.*2.*3.1415)
        e1 = sim.calculate_energy()
        self.assertLess(math.fabs((e0-e1)/e1),2.4e-8)

    def test_whfast_veryhyperbolic(self):
        sim = rebound.Simulation()
        sim.add(m=1.)
        sim.add(m=0.,x=1.,vy=100000.)
        sim.integrator = "whfast"
        sim.ri_whfast.coordinates = "jacobi"
        sim.dt = 1.234567
        sim.step()
        y = sim.particles[1].y
        ys = 1.234567*100000.
        self.assertAlmostEqual((y-ys)/ys, 0., delta=1e-15)

    def test_whfast_hyperbolic(self):
        sim = rebound.Simulation()
        sim.add(m=1.)
        sim.add(m=1e-3, a=-1.,e=2.5)
        sim.integrator = "whfast"
        sim.ri_whfast.coordinates = "jacobi"
        e0 = sim.calculate_energy()
        yr = -sim.particles[1].P
        sim.dt = 0.00512*yr
        sim.integrate(1e2*yr)
        e1 = sim.calculate_energy()
        self.assertLess(math.fabs((e0-e1)/e1),1e-14)
    
    def test_whfast(self):
        sim = rebound.Simulation()
        sim.add(m=1.)
        sim.add(m=1e-3, a=1.,e=.1)
        sim.add(m=1e-3, a=3.,e=0.1)
        sim.integrator = "whfast"
        sim.ri_whfast.coordinates = "jacobi"
        jupyr = 2.*math.pi
        sim.dt = 0.005123*jupyr
        e0 = sim.calculate_energy()
        sim.integrate(1e3*jupyr)
        self.assertNotEqual(e0,0.)
        e1 = sim.calculate_energy()
        self.assertLess(math.fabs((e0-e1)/e1),1.2e-8)
    
    def test_whfast_nosafemode(self):
        sim = rebound.Simulation()
        sim.add(m=1.)
        sim.add(m=1e-3, a=1.,e=.1)
        sim.add(m=1e-3, a=3.,e=0.1)
        sim.integrator = "whfast"
        sim.ri_whfast.coordinates = "democraticheliocentric"
        sim.ri_whfast.safe_mode = 0
        jupyr = 2.*math.pi
        sim.dt = 0.005123*jupyr
        e0 = sim.calculate_energy()
        sim.integrate(1e3*jupyr)
        self.assertNotEqual(e0,0.)
        e1 = sim.calculate_energy()
        self.assertLess(math.fabs((e0-e1)/e1),3e-8)
    
    # Democratic Heliocentric
    def test_order_doesnt_matter_tp0(self):
        sim = rebound.Simulation()
        sim.add(m=1)
        sim.add(m=1e-3,a=1.1,e=0.1,primary=sim.particles[0])
        sim.add(m=5e-3,a=1.0,e=0.1,primary=sim.particles[0])

        sim.integrator = "whfast"
        sim.ri_whfast.coordinates = "democraticheliocentric"
        sim.dt = 1e-2*2.*3.14
        sim.N_active = 1
        sim.testparticle_type = 0

        sim.integrate(1000)
        
        o1 = sim.particles[1].calculate_orbit(primary=sim.particles[0])
        o2 = sim.particles[2].calculate_orbit(primary=sim.particles[0])
        

        sim = rebound.Simulation()
        sim.add(m=1)
        sim.add(m=5e-3,a=1.0,e=0.1,primary=sim.particles[0])
        sim.add(m=1e-3,a=1.1,e=0.1,primary=sim.particles[0])

        sim.integrator = "whfast"
        sim.ri_whfast.coordinates = "democraticheliocentric"
        sim.dt = 1e-2*2.*3.14
        sim.N_active = 1
        sim.testparticle_type = 0

        sim.integrate(1000)
        
        o3 = sim.particles[1].calculate_orbit(primary=sim.particles[0])
        o4 = sim.particles[2].calculate_orbit(primary=sim.particles[0])

        self.assertLess(abs(o1.e-o4.e),2e-16)
        self.assertLess(abs(o2.e-o3.e),2e-16)
        self.assertLess(abs(o1.a-o4.a),2e-16)
        self.assertLess(abs(o2.a-o3.a),2e-16)
    
    def test_order_doesnt_matter_tp1(self):
        sim = rebound.Simulation()
        sim.add(m=1)
        sim.add(m=1e-3,a=1.1,e=0.1,primary=sim.particles[0])
        sim.add(m=5e-3,a=1.0,e=0.1,primary=sim.particles[0])

        sim.integrator = "whfast"
        sim.ri_whfast.coordinates = "democraticheliocentric"
        sim.dt = 1e-2*2.*3.14
        sim.N_active = 1
        sim.testparticle_type = 1

        sim.integrate(1000)
        
        o1 = sim.particles[1].calculate_orbit(primary=sim.particles[0])
        o2 = sim.particles[2].calculate_orbit(primary=sim.particles[0])
        

        sim = rebound.Simulation()
        sim.add(m=1)
        sim.add(m=5e-3,a=1.0,e=0.1,primary=sim.particles[0])
        sim.add(m=1e-3,a=1.1,e=0.1,primary=sim.particles[0])

        sim.integrator = "whfast"
        sim.ri_whfast.coordinates = "democraticheliocentric"
        sim.dt = 1e-2*2.*3.14
        sim.N_active = 1
        sim.testparticle_type = 1

        sim.integrate(1000)
        
        o3 = sim.particles[1].calculate_orbit(primary=sim.particles[0])
        o4 = sim.particles[2].calculate_orbit(primary=sim.particles[0])

        self.assertLess(abs(o1.e-o4.e),4e-14)
        self.assertLess(abs(o2.e-o3.e),5e-14)
        self.assertLess(abs(o1.a-o4.a),7e-14)
        self.assertLess(abs(o2.a-o3.a),4e-14)

    def test_testparticle_type_one(self):
        # Testparticle type 1 with only 1 semi-active particle
        # should be the same as having only active particles
        sim = rebound.Simulation()
        sim.add(m=1)
        sim.add(m=1e-3,a=1.1,e=0.1,primary=sim.particles[0])
        sim.add(m=5e-3,a=1.0,e=0.1,primary=sim.particles[0])

        sim.integrator = "whfast"
        sim.ri_whfast.coordinates = "democraticheliocentric"
        sim.dt = 1e-2*2.*3.14
        sim.N_active = sim.N-1
        sim.testparticle_type = 1

        sim.integrate(1000)
        
        o1 = sim.particles[1].calculate_orbit(primary=sim.particles[0])
        o2 = sim.particles[2].calculate_orbit(primary=sim.particles[0])
        

        sim = rebound.Simulation()
        sim.add(m=1)
        sim.add(m=1e-3,a=1.1,e=0.1,primary=sim.particles[0])
        sim.add(m=5e-3,a=1.0,e=0.1,primary=sim.particles[0])

        sim.integrator = "whfast"
        sim.ri_whfast.coordinates = "democraticheliocentric"
        sim.dt = 1e-2*2.*3.14

        sim.integrate(1000)
        
        o3 = sim.particles[1].calculate_orbit(primary=sim.particles[0])
        o4 = sim.particles[2].calculate_orbit(primary=sim.particles[0])

        self.assertLess(abs(o1.e-o3.e),2e-16)
        self.assertLess(abs(o2.e-o4.e),2e-16)
        self.assertLess(abs(o1.a-o3.a),2e-16)
        self.assertLess(abs(o2.a-o4.a),2e-16)

    def test_whfasthelio_outersolarsystem(self):
        sim = rebound.Simulation()
        rebound.data.add_outer_solar_system(sim)
        sim.integrator = "whfast"
        sim.ri_whfast.coordinates = "democraticheliocentric"
        sim.dt = 0.005*12.*2.*3.1415   # ~ 1/200 of a jupiter year
        e0 = sim.calculate_energy()
        sim.integrate(1000.*2.*3.1415)
        e1 = sim.calculate_energy()
        self.assertLess(math.fabs((e0-e1)/e1),1.2e-8)
    
    def test_whfasthelio_veryhyperbolic(self):
        sim = rebound.Simulation()
        sim.add(m=1.)
        sim.add(m=0.,x=1.,vy=100000.)
        sim.integrator = "whfast"
        sim.ri_whfast.coordinates = "democraticheliocentric"
        sim.dt = 1.234567
        sim.step()
        y = sim.particles[1].y
        ys = 1.234567*100000.
        self.assertAlmostEqual((y-ys)/ys, 0., delta=1e-15)

    def test_whfasthelio_hyperbolic(self):
        sim = rebound.Simulation()
        sim.add(m=1.)
        sim.add(m=1e-3, a=-1.,e=2.5)
        sim.integrator = "whfast"
        sim.ri_whfast.coordinates = "democraticheliocentric"
        e0 = sim.calculate_energy()
        yr = -sim.particles[1].P
        sim.dt = 0.00512*yr
        sim.integrate(1e2*yr)
        e1 = sim.calculate_energy()
        self.assertLess(math.fabs((e0-e1)/e1),4.5e-8)
    
    def test_whfasthelio(self):
        sim = rebound.Simulation()
        sim.add(m=1.)
        sim.add(m=1e-3, a=1.,e=.1)
        sim.add(m=1e-3, a=3.,e=0.1)
        sim.integrator = "whfast"
        sim.ri_whfast.coordinates = "democraticheliocentric"
        jupyr = 2.*math.pi
        sim.dt = 0.005123*jupyr
        e0 = sim.calculate_energy()
        sim.integrate(1e3*jupyr)
        self.assertNotEqual(e0,0.)
        e1 = sim.calculate_energy()
        self.assertLess(math.fabs((e0-e1)/e1),2.9e-8)
    
    def test_whfasthelio_orderdoesnotmatter(self):
        jupyr = 2.*math.pi

        sim1 = rebound.Simulation()
        sim1.add(m=1.)
        sim1.add(m=1e-3, a=1.,e=0.1,primary=sim1.particles[0])
        sim1.add(m=1e-3, a=3.,e=0.1,primary=sim1.particles[0])
        sim1.integrator = "whfast"
        sim1.ri_whfast.coordinates = "democraticheliocentric"
        sim1.dt = 0.005123*jupyr
        sim1.integrate(1e0*jupyr)
        sim2 = rebound.Simulation()
        sim2.add(m=1.)
        sim2.add(m=1e-3, a=3.,e=0.1,primary=sim2.particles[0])
        sim2.add(m=1e-3, a=1.,e=0.1,primary=sim2.particles[0])
        sim2.integrator = "whfast"
        sim2.ri_whfast.coordinates = "democraticheliocentric"
        sim2.dt = 0.005123*jupyr
        sim2.integrate(1e0*jupyr)
        
        self.assertAlmostEqual(sim1.particles[1].x, sim2.particles[2].x, delta=1e-14)
        self.assertAlmostEqual(sim1.particles[1].y, sim2.particles[2].y, delta=1e-14)
        self.assertAlmostEqual(sim1.particles[2].x, sim2.particles[1].x, delta=1e-14)
        self.assertAlmostEqual(sim1.particles[2].y, sim2.particles[1].y, delta=1e-14)
        
    
    def test_whfasthelio_nosafemode(self):
        sim = rebound.Simulation()
        sim.add(m=1.)
        sim.add(m=1e-3, a=1.,e=.1)
        sim.add(m=1e-3, a=3.,e=0.1)
        sim.integrator = "whfast"
        sim.ri_whfast.coordinates = "democraticheliocentric"
        sim.ri_whfast.safe_mode = 0
        jupyr = 2.*math.pi
        sim.dt = 0.005123*jupyr
        e0 = sim.calculate_energy()
        sim.integrate(1e3*jupyr)
        self.assertNotEqual(e0,0.)
        e1 = sim.calculate_energy()
        self.assertLess(math.fabs((e0-e1)/e1),2.9e-8)


class TestIntegratorWHFastBackAndForth(unittest.TestCase):
    def test_whfast_hyperbolic(self):
        sim = rebound.Simulation()
        sim.add(m=1.)
        sim.add(m=1e-3, a=-1.,e=2.5,omega=0.3,Omega=0.4,inc=0.1)
        x0 = sim.particles[1].x;
        v0 = sim.particles[1].vx;
        sim.integrator = "whfast"
        e0 = sim.calculate_energy()
        yr = -sim.particles[1].P
        sim.dt = 0.0512*yr
        for i in range(100):
            sim.step()
        sim.dt *= -1
        for i in range(100):
            sim.step()
        x1 = sim.particles[1].x;
        v1 = sim.particles[1].vx;
        e1 = sim.calculate_energy()
        self.assertLess(math.fabs((e0-e1)/e1),1e-14)
        self.assertLess(math.fabs((x0-x1)/x1),1e-14)
        self.assertLess(math.fabs((v0-v1)/v1),1e-14)
    
    def test_whfast_eccentric(self):
        sim = rebound.Simulation()
        sim.add(m=1.)
        sim.add(m=1e-3, a=1.4,e=0.0125,omega=0.3,Omega=0.4,inc=0.1)
        x0 = sim.particles[1].x;
        v0 = sim.particles[1].vx;
        sim.integrator = "whfast"
        e0 = sim.calculate_energy()
        yr = sim.particles[1].P
        sim.dt = 0.0512*yr
        for i in range(100):
            sim.step()
        sim.dt *= -1
        for i in range(100):
            sim.step()
        x1 = sim.particles[1].x;
        v1 = sim.particles[1].vx;
        e1 = sim.calculate_energy()
        self.assertLess(math.fabs((e0-e1)/e1),1e-13)
        self.assertLess(math.fabs((x0-x1)/x1),1e-13)
        self.assertLess(math.fabs((v0-v1)/v1),1e-13)
    


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import math
from .particle import Particle
from itertools import cycle

def OrbitPlot(sim, figsize=None, fancy=False, slices=0, xlim=None, ylim=None, unitlabel=None, color=False, periastron=False, orbit_type="trail", lw=1., plotparticles=[], primary=None, Narc=128):
    """
    Convenience function for plotting instantaneous orbits.

    Parameters
    ----------
    figsize         : tuple of float, optional
        Tuple defining the figure size (default: (5,5))
    fancy           : bool (default: False)
        Changes various settings to create a fancy looking plot
    slices          : float, optional
        Default is 0, showing the orbits in the xy plane only. Set to a value between 0 and 1 to create three plots, showing the orbits from different directions. The value corresponds to the size of the additional plots relative to the main plot.
    xlim            : tuple of float, optional           
        Limits for x axes (default: None = automatically determined)
    ylim            : tuple of float, optional           
        Limits for y axes (default: None = automatically determined)
    unitlabel       : str, optional          
        String describing the units, shown on axis labels (default: None)
    color           : bool, str or list, optional            
        By default plots are black and white. If set to True, plots use a color cycle. If a string or list of strings, e.g. ['red', 'cyan'], will cycle between passed colors.
    periastron  : bool, optional            
        Draw a marker at periastron (default: False)
    orbit_type       : str, optional
        This argument determines the type of orbit show. By default it shows the orbit as a trailing and fading line ("trail"). Other object are: "solid", None.
    lw              : float, optional           
        Linewidth used in plots (default: 1.)
    plotparticles   : list, optional
        List of particles to plot. Can be a list of any valid keys for accessing sim.particles, i.e., integer indices or hashes (default: plot all particles)
    primary         : rebound.Particle, optional
        Primary to use for the osculating orbit (default: Jacobi center of mass)
    Narc            : int, optional
        Number of points used in an orbit. Increase this number for highly eccentric orbits. (default: 128)

    Returns
    -------
    fig, ax_main, (ax_top, ax_right)
        The function return the matplotlib figure as well as the axes (three axes if slices>0.)

    Examples
    --------
    The following example illustrates a typical use case.

    >>> sim = rebound.Simulation()
    >>> sim.add(m=1)
    >>> sim.add(a=1)
    >>> fig, ax_main = rebound.OrbitPlot(sim)
    >>> fig.savefig("image.png") # save figure to file
    >>> fig.show() # show figure on screen

    """
    try:
        import matplotlib.pyplot as plt
        from matplotlib import gridspec
        from mpl_toolkits.axes_grid1 import make_axes_locatable
        import numpy as np
    except:
        raise ImportError("Error importing matplotlib and/or numpy. Plotting functions not available. If running from within a jupyter notebook, try calling '%matplotlib inline' beforehand.")
    if unitlabel is not None:
        unitlabel = " " + unitlabel
    else:
        unitlabel = ""
    if figsize is None:
        if slices>0.:
            figsize = (8,8)
        else:
            figsize = (5,5)
    
    fig = plt.figure(figsize=figsize)
    ax_main = plt.subplot(111,aspect="equal")
    ax_main.set_xlabel("x"+unitlabel)
    ax_main.set_ylabel("y"+unitlabel)

    if slices>0.:
        divider = make_axes_locatable(ax_main)
        divider.set_aspect(True)
        ax_top   = divider.append_axes("top",  size="%.2f%%"%(100.*slices), sharex=ax_main)
        ax_top.set_aspect('equal', adjustable='datalim')
        ax_right = divider.append_axes("right", size="%.2f%%"%(100.*slices), sharey=ax_main)
        ax_right.set_aspect('equal', adjustable='datalim')
      
        plt.setp(ax_top.get_xticklabels(), visible=False)
        plt.setp(ax_top.get_xticklines(), visible=False)
        ax_top.set_ylabel("z"+unitlabel)
        
        plt.setp(ax_right.get_yticklabels(), visible=False)
        plt.setp(ax_right.get_yticklines(), visible=False)
        ax_right.set_xlabel("z"+unitlabel)

    OrbitPlotOneSlice(sim, ax_main, Narc=Narc, color=color, periastron=periastron, orbit_type=orbit_type, lw=lw, axes="xy",fancy=fancy, plotparticles=plotparticles, primary=primary)

    if slices>0.:
        OrbitPlotOneSlice(sim, ax_right, Narc=Narc, color=color, periastron=periastron, orbit_type=orbit_type, lw=lw,fancy=fancy, axes="zy", plotparticles=plotparticles, primary=primary)
        OrbitPlotOneSlice(sim, ax_top, Narc=Narc, color=color, periastron=periastron, orbit_type=orbit_type, lw=lw,fancy=fancy, axes="xz", plotparticles=plotparticles, primary=primary)

    if xlim is not None:
        ax_main.set_xlim(xlim)
    if ylim is not None:
        ax_main.set_ylim(ylim)


    if fancy:
        ax_main.apply_aspect()
        if slices>0.:
            ax_top.apply_aspect()
            ax_right.apply_aspect()
        ax_main.autoscale(False)
        if slices>0.:
            ax_top.autoscale(False)
            ax_right.autoscale(False)
        OrbitPlotAddFancyStars(ax_main,lw)
        if slices>0.:
            OrbitPlotAddFancyStars(ax_top,lw,slices)
            OrbitPlotAddFancyStars(ax_right,lw,slices)
    if slices>0.:
        return fig, ax_main, ax_top, ax_right
    else:
        return fig, ax_main

def get_color(color):
    """
    Takes a string for a color name defined in matplotlib and returns of a 3-tuple of RGB values.
    Will simply return passed value if it's a tuple of length three.

    Parameters
    ----------
    color   : str
        Name of matplotlib color to calculate RGB values for.
    """

    if isinstance(color, tuple) and len(color) == 3: # already a tuple of RGB values
        return color

    try:
        import matplotlib.colors as mplcolors
    except:
        raise ImportError("Error importing matplotlib. If running from within a jupyter notebook, try calling '%matplotlib inline' beforehand.")
   
    try:
        hexcolor = mplcolors.cnames[color]
    except KeyError:
        raise AttributeError("Color not recognized in matplotlib.")

    hexcolor = hexcolor.lstrip('#')
    lv = len(hexcolor)
    return tuple(int(hexcolor[i:i + lv // 3], 16)/255. for i in range(0, lv, lv // 3)) # tuple of rgb values

def fading_line(x, y, color='black', alpha_initial=1., alpha_final=0., glow=False, **kwargs):
    """
    Returns a matplotlib LineCollection connecting the points in the x and y lists, with a single color and alpha varying from alpha_initial to alpha_final along the line.
    Can pass any kwargs you can pass to LineCollection, like linewidgth.

    Parameters
    ----------
    x       : list or array of floats for the positions on the (plot's) x axis
    y       : list or array of floats for the positions on the (plot's) y axis
    color   : matplotlib color for the line. Can also pass a 3-tuple of RGB values (default: 'black')
    alpha_initial:  Limiting value of alpha to use at the beginning of the arrays.
    alpha_final:    Limiting value of alpha to use at the end of the arrays.
    """
    try:
        from matplotlib.collections import LineCollection
        from matplotlib.colors import LinearSegmentedColormap
        import numpy as np
    except:
        raise ImportError("Error importing matplotlib and/or numpy. Plotting functions not available. If running from within a jupyter notebook, try calling '%matplotlib inline' beforehand.")


    if "lw" not in kwargs:
        kwargs["lw"] = 1
    lw = kwargs["lw"]

    if glow:
        kwargs["lw"] = 1*lw
        fl1 = fading_line(x, y, color, alpha_initial, alpha_final, glow=False, **kwargs)
        kwargs["lw"] = 2*lw
        alpha_initial *= 0.5
        alpha_final *= 0.5
        fl2 = fading_line(x, y, color, alpha_initial, alpha_final, glow=False, **kwargs)
        kwargs["lw"] = 6*lw
        alpha_initial *= 0.5
        alpha_final *= 0.5
        fl3 = fading_line(x, y, color, alpha_initial, alpha_final, glow=False, **kwargs)
        return [fl3,fl2,fl1]

    color = get_color(color)
    cdict = {'red': ((0.,color[0],color[0]),(1.,color[0],color[0])),
             'green': ((0.,color[1],color[1]),(1.,color[1],color[1])),
             'blue': ((0.,color[2],color[2]),(1.,color[2],color[2])),
             'alpha': ((0.,alpha_initial, alpha_initial), (1., alpha_final, alpha_final))}
    
    Npts = len(x)
    if len(y) != Npts:
        raise AttributeError("x and y must have same dimension.")
   
    segments = np.zeros((Npts-1,2,2))
    segments[0][0] = [x[0], y[0]]
    for i in range(1,Npts-1):
        pt = [x[i], y[i]]
        segments[i-1][1] = pt
        segments[i][0] = pt 
    segments[-1][1] = [x[-1], y[-1]]

    individual_cm = LinearSegmentedColormap('indv1', cdict)
    lc = LineCollection(segments, cmap=individual_cm, **kwargs)
    lc.set_array(np.linspace(0.,1.,len(segments)))
    return lc

def OrbitPlotOneSlice(sim, ax, Narc=128, color=False, periastron=False, orbit_type="trial", lw=1., axes="xy", plotparticles=[], primary=None, fancy=False):
    import matplotlib.pyplot as plt
    from matplotlib.collections import LineCollection
    from matplotlib.colors import LinearSegmentedColormap
    import numpy as np
    import random

    #ax.set_aspect("equal")
    p_orb_pairs = []
    if not plotparticles:
        plotparticles = range(1, sim.N_real)
    for i in plotparticles:
        p = sim.particles[i]
        p_orb_pairs.append((p, p.calculate_orbit(primary=primary)))


    if color:
        if color == True:
            colors = [(1.,0.,0.),(0.,0.75,0.75),(0.75,0.,0.75),(0.75, 0.75, 0,),(0., 0., 0.),(0., 0., 1.),(0., 0.5, 0.)]
        if isinstance(color, str):
            colors = [get_color(color)]
        if isinstance(color, list):
            colors = []
            for c in color:
                colors.append(get_color(c))
    else:
        if fancy:
            colors = [(181./206.,66./206.,191./206.)]
        else:
            colors = ["black"]
    coloriterator = cycle(colors)

    coords = {'x':0, 'y':1, 'z':2}
    axis0 = coords[axes[0]]
    axis1 = coords[axes[1]]
   
    prim = sim.particles[0] if primary is None else primary 
    if fancy:
        sun = (256./256.,256./256.,190./256.)
        opa = 0.035
        size = 6000.
        for i in range(100):
            ax.scatter(getattr(prim,axes[0]),getattr(prim,axes[1]), alpha=opa, s=size*lw, facecolor=sun, edgecolor=None, zorder=3)
            size *= 0.95
        ax.scatter(getattr(prim,axes[0]),getattr(prim,axes[1]), s=size*lw, facecolor=sun, edgecolor=None, zorder=3)
    else:
        ax.scatter(getattr(prim,axes[0]),getattr(prim,axes[1]), marker="*", s=35*lw, facecolor="black", edgecolor=None, zorder=3)
    
    proj = {}
    for p, o in p_orb_pairs:
        prim = p.jacobi_com if primary is None else primary 

        colori = next(coloriterator)

        if fancy:
            ax.scatter(getattr(p,axes[0]), getattr(p,axes[1]), s=25*lw, facecolor=colori, edgecolor=None, zorder=3)
        else:
            ax.scatter(getattr(p,axes[0]), getattr(p,axes[1]), s=25*lw, facecolor="black", edgecolor=None, zorder=3)
       
        if orbit_type is not None:
            alpha_final = 0. if orbit_type=="trail" else 1. # fade to 0 with trail

            hyperbolic = o.a < 0. # Boolean for whether orbit is hyperbolic
            if hyperbolic is False:
                pts = np.array(p.sample_orbit(Npts=Narc+1, primary=prim))
                proj['x'],proj['y'],proj['z'] = [pts[:,i] for i in range(3)]
                lc = fading_line(proj[axes[0]], proj[axes[1]], colori, alpha_final=alpha_final, lw=lw, glow=fancy)
                if type(lc) is list:
                    for l in lc:
                        ax.add_collection(l)
                else:
                    ax.add_collection(lc)

            else:
                pts = np.array(p.sample_orbit(Npts=Narc+1, primary=prim, useTrueAnomaly=False))
                # true anomaly stays close to limiting value and switches quickly at pericenter for hyperbolic orbit, so use mean anomaly
                proj['x'],proj['y'],proj['z'] = [pts[:,i] for i in range(3)]
                lc = fading_line(proj[axes[0]], proj[axes[1]], colori, alpha_final=alpha_final, lw=lw, glow=fancy)
                if type(lc) is list:
                    for l in lc:
                        ax.add_collection(l)
                else:
                    ax.add_collection(lc)
          
                alpha = 0.2 if orbit_type=="trail" else 1.
                pts = np.array(p.sample_orbit(Npts=Narc+1, primary=prim, trailing=False, useTrueAnomaly=False))
                proj['x'],proj['y'],proj['z'] = [pts[:,i] for i in range(3)]
                lc = fading_line(proj[axes[0]], proj[axes[1]], colori, alpha_initial=alpha, alpha_final=alpha, lw=lw, glow=fancy)
                if type(lc) is list:
                    for l in lc:
                        ax.add_collection(l)
                else:
                    ax.add_collection(lc)

        if periastron:
            newp = Particle(a=o.a, f=0., inc=o.inc, omega=o.omega, Omega=o.Omega, e=o.e, m=p.m, primary=prim, simulation=sim)
            ax.plot([getattr(prim,axes[0]), getattr(newp,axes[0])], [getattr(prim,axes[1]), getattr(newp,axes[1])], linestyle="dotted", c=colori, zorder=1, lw=lw)
            ax.scatter([getattr(newp,axes[0])],[getattr(newp,axes[1])], marker="o", s=5.*lw, facecolor="none", edgecolor=colori, zorder=1)


def OrbitPlotAddFancyStars(ax,lw,slices=1.):
    import numpy as np
    # safe the current random seed to restore later
    os = np.random.get_state()
    # always produce the same stars
    np.random.seed(1) 

    ax.set_facecolor((0.,0.,0.))
    for pos in ['top', 'bottom', 'right', 'left']:
        ax.spines[pos].set_edgecolor((0.3,0.3,0.3))
    
    starcolor = (1.,1.,1.)
    starsurfacedensity = 0.8

    area = np.sqrt(np.sum(np.square(ax.transAxes.transform([1.,1.]) - ax.transAxes.transform([0.,0.]))))*slices
    nstars = int(starsurfacedensity*area)

    #small stars
    xy = np.random.uniform(size=(nstars,2))
    ax.scatter(xy[:,0],xy[:,1], transform=ax.transAxes, alpha=0.05, s=8*lw, facecolor=starcolor, edgecolor=None, zorder=3)
    ax.scatter(xy[:,0],xy[:,1], transform=ax.transAxes, alpha=0.1, s=4*lw, facecolor=starcolor, edgecolor=None, zorder=3)
    ax.scatter(xy[:,0],xy[:,1], transform=ax.transAxes, alpha=0.2, s=0.5*lw, facecolor=starcolor, edgecolor=None, zorder=3)
    
    #large stars
    xy = np.random.uniform(size=(nstars//4,2))
    ax.scatter(xy[:,0],xy[:,1], transform=ax.transAxes, alpha=0.1, s=15*lw, facecolor=starcolor, edgecolor=None, zorder=3)
    ax.scatter(xy[:,0],xy[:,1], transform=ax.transAxes, alpha=0.1, s=5*lw, facecolor=starcolor, edgecolor=None, zorder=3)
    ax.scatter(xy[:,0],xy[:,1], transform=ax.transAxes, alpha=0.5, s=2*lw, facecolor=starcolor, edgecolor=None, zorder=3)

    np.random.set_state(os)






# -*- coding: utf-8 -*-
"""Automatically generate citations for a simulation."""

def cite(sim):
    txt = """Simulations in this paper made use of the REBOUND N-body code \citep{rebound}. """
    bib = """@ARTICLE{rebound,
       author = {{Rein}, H. and {Liu}, S. -F.},
        title = "{REBOUND: an open-source multi-purpose N-body code for collisional dynamics}",
      journal = {\\aap},
     keywords = {methods: numerical, planets and satellites: rings, protoplanetary disks, Astrophysics - Earth and Planetary Astrophysics, Astrophysics - Instrumentation and Methods for Astrophysics, Mathematics - Dynamical Systems, Physics - Computational Physics},
         year = 2012,
        month = jan,
       volume = {537},
          eid = {A128},
        pages = {A128},
          doi = {10.1051/0004-6361/201118085},
archivePrefix = {arXiv},
       eprint = {1110.4876},
 primaryClass = {astro-ph.EP},
       adsurl = {https://ui.adsabs.harvard.edu/abs/2012A&A...537A.128R},
      adsnote = {Provided by the SAO/NASA Astrophysics Data System}
}
"""
    if sim.integrator == "ias15":
        txt += """The simulations were integrated using IAS15, a 15th order Gauss-Radau integrator \citep{reboundias15}. """
        bib += """@ARTICLE{reboundias15,
       author = {{Rein}, Hanno and {Spiegel}, David S.},
        title = "{IAS15: a fast, adaptive, high-order integrator for gravitational dynamics, accurate to machine precision over a billion orbits}",
      journal = {\\mnras},
     keywords = {gravitation, methods: numerical, planets and satellites: dynamical evolution and stability, Astrophysics - Earth and Planetary Astrophysics, Astrophysics - Instrumentation and Methods for Astrophysics, Astrophysics - Solar and Stellar Astrophysics, Mathematics - Numerical Analysis},
         year = 2015,
        month = jan,
       volume = {446},
       number = {2},
        pages = {1424-1437},
          doi = {10.1093/mnras/stu2164},
archivePrefix = {arXiv},
       eprint = {1409.4779},
 primaryClass = {astro-ph.EP},
       adsurl = {https://ui.adsabs.harvard.edu/abs/2015MNRAS.446.1424R},
      adsnote = {Provided by the SAO/NASA Astrophysics Data System}
}
"""
    if sim.integrator == "whfast":
        txt += """The simulations were integrated using WHFast, a symplectic Wisdom-Holman integrator \citep{reboundwhfast,wh}. """
        bib += """@ARTICLE{reboundwhfast,
       author = {{Rein}, Hanno and {Tamayo}, Daniel},
        title = "{WHFAST: a fast and unbiased implementation of a symplectic Wisdom-Holman integrator for long-term gravitational simulations}",
      journal = {\\mnras},
     keywords = {gravitation, methods: numerical, planets and satellites: dynamical evolution and stability, Astrophysics - Earth and Planetary Astrophysics, Astrophysics - Instrumentation and Methods for Astrophysics, Mathematics - Numerical Analysis, Nonlinear Sciences - Chaotic Dynamics, Physics - Computational Physics},
         year = 2015,
        month = sep,
       volume = {452},
       number = {1},
        pages = {376-388},
          doi = {10.1093/mnras/stv1257},
archivePrefix = {arXiv},
       eprint = {1506.01084},
 primaryClass = {astro-ph.EP},
       adsurl = {https://ui.adsabs.harvard.edu/abs/2015MNRAS.452..376R},
      adsnote = {Provided by the SAO/NASA Astrophysics Data System}
}
"""
        bib += """@ARTICLE{wh,
       author = {{Wisdom}, Jack and {Holman}, Matthew},
        title = "{Symplectic maps for the N-body problem.}",
      journal = {\aj},
     keywords = {Many Body Problem, Planetary Evolution, Pluto (Planet), Astronomical Maps, Gravitational Effects, Physics (General)},
         year = 1991,
        month = oct,
       volume = {102},
        pages = {1528-1538},
          doi = {10.1086/115978},
       adsurl = {https://ui.adsabs.harvard.edu/abs/1991AJ....102.1528W},
      adsnote = {Provided by the SAO/NASA Astrophysics Data System}
}
"""
        if sim.ri_whfast.kernel != "default":
            txt += """A high order kernel was used in WHFast to improved the accuracy of the integrations \citep{reboundhighorder}. """
            bib += """@ARTICLE{reboundhighorder,
           author = {{Rein}, Hanno and {Tamayo}, Daniel and {Brown}, Garett},
            title = "{High-order symplectic integrators for planetary dynamics and their implementation in REBOUND}",
          journal = {\mnras},
         keywords = {gravitation, methods: numerical, planets and satellites: dynamical evolution and stability, Astrophysics - Earth and Planetary Astrophysics, Astrophysics - Instrumentation and Methods for Astrophysics, Physics - Computational Physics},
             year = 2019,
            month = nov,
           volume = {489},
           number = {4},
            pages = {4632-4640},
              doi = {10.1093/mnras/stz2503},
    archivePrefix = {arXiv},
           eprint = {1907.11335},
     primaryClass = {astro-ph.EP},
           adsurl = {https://ui.adsabs.harvard.edu/abs/2019MNRAS.489.4632R},
          adsnote = {Provided by the SAO/NASA Astrophysics Data System}
    }
    """


    
    if sim.integrator == "mercurius":
        txt += """The simulations were integrated using the hybrid symplectic MERCURIUS integrator \citep{reboundmercurius}. """
        bib += """@ARTICLE{reboundmercurius,
       author = {{Rein}, Hanno and {Hernandez}, David M. and {Tamayo}, Daniel and
         {Brown}, Garett and {Eckels}, Emily and {Holmes}, Emma and
         {Lau}, Michelle and {Leblanc}, R{\'e}jean and {Silburt}, Ari},
        title = "{Hybrid symplectic integrators for planetary dynamics}",
      journal = {\mnras},
     keywords = {gravitation, methods: numerical, planets and satellites: dynamical evolution and stability, Astrophysics - Earth and Planetary Astrophysics, Astrophysics - Instrumentation and Methods for Astrophysics, Mathematics - Dynamical Systems},
         year = 2019,
        month = jun,
       volume = {485},
       number = {4},
        pages = {5490-5497},
          doi = {10.1093/mnras/stz769},
archivePrefix = {arXiv},
       eprint = {1903.04972},
 primaryClass = {astro-ph.EP},
       adsurl = {https://ui.adsabs.harvard.edu/abs/2019MNRAS.485.5490R},
      adsnote = {Provided by the SAO/NASA Astrophysics Data System}
}
"""

    if sim.integrator == "janus":
        txt += """The simulations were integrated using the time-reversible JANUS integrator \citep{reboundjanus}. """
        bib += """@ARTICLE{reboundjanus,
       author = {{Rein}, Hanno and {Tamayo}, Daniel},
        title = "{JANUS: a bit-wise reversible integrator for N-body dynamics}",
      journal = {\mnras},
     keywords = {gravitation, methods: numerical, planets and satellites: dynamical evolution and stability, Astrophysics - Instrumentation and Methods for Astrophysics, Astrophysics - Cosmology and Nongalactic Astrophysics, Astrophysics - Earth and Planetary Astrophysics},
         year = 2018,
        month = jan,
       volume = {473},
       number = {3},
        pages = {3351-3357},
          doi = {10.1093/mnras/stx2479},
archivePrefix = {arXiv},
       eprint = {1704.07715},
 primaryClass = {astro-ph.IM},
       adsurl = {https://ui.adsabs.harvard.edu/abs/2018MNRAS.473.3351R},
      adsnote = {Provided by the SAO/NASA Astrophysics Data System}
}
"""

    if sim.integrator == "sei":
        txt += """The simulations were integrated using the Symplectic Epicycle Integrator (SEI) \citep{reboundsei}. """
        bib += """@ARTICLE{reboundsei,
       author = {{Rein}, Hanno and {Tremaine}, Scott},
        title = "{Symplectic integrators in the shearing sheet}",
      journal = {\mnras},
     keywords = {methods: numerical, celestial mechanics, planets and satellites: dynamical evolution and stability, planets and satellites: formation, planets and satellites: rings, Astrophysics - Earth and Planetary Astrophysics, Astrophysics - Galaxy Astrophysics, Astrophysics - Instrumentation and Methods for Astrophysics, Mathematics - Numerical Analysis},
         year = 2011,
        month = aug,
       volume = {415},
       number = {4},
        pages = {3168-3176},
          doi = {10.1111/j.1365-2966.2011.18939.x},
archivePrefix = {arXiv},
       eprint = {1103.1376},
 primaryClass = {astro-ph.EP},
       adsurl = {https://ui.adsabs.harvard.edu/abs/2011MNRAS.415.3168R},
      adsnote = {Provided by the SAO/NASA Astrophysics Data System}
}
"""

    if sim.integrator == "saba":
        txt += """The simulations were integrated using the SABA Integrator \citep{reboundhighorder,saba}. """
        bib += """@ARTICLE{reboundhighorder,
       author = {{Rein}, Hanno and {Tamayo}, Daniel and {Brown}, Garett},
        title = "{High-order symplectic integrators for planetary dynamics and their implementation in REBOUND}",
      journal = {\mnras},
     keywords = {gravitation, methods: numerical, planets and satellites: dynamical evolution and stability, Astrophysics - Earth and Planetary Astrophysics, Astrophysics - Instrumentation and Methods for Astrophysics, Physics - Computational Physics},
         year = 2019,
        month = nov,
       volume = {489},
       number = {4},
        pages = {4632-4640},
          doi = {10.1093/mnras/stz2503},
archivePrefix = {arXiv},
       eprint = {1907.11335},
 primaryClass = {astro-ph.EP},
       adsurl = {https://ui.adsabs.harvard.edu/abs/2019MNRAS.489.4632R},
      adsnote = {Provided by the SAO/NASA Astrophysics Data System}
}
"""
        bib += """@ARTICLE{saba,
       author = {{Laskar}, Jacques and {Robutel}, Philippe},
        title = "{High order symplectic integrators for perturbed Hamiltonian systems}",
      journal = {Celestial Mechanics and Dynamical Astronomy},
     keywords = {SYMPLECTIC INTEGRATORS, HAMILTONIAN SYSTEMS, PLANETARY MOTION, LIE ALGEBRA, Astrophysics},
         year = 2001,
        month = jul,
       volume = {80},
       number = {1},
        pages = {39-62},
archivePrefix = {arXiv},
       eprint = {astro-ph/0005074},
 primaryClass = {astro-ph},
       adsurl = {https://ui.adsabs.harvard.edu/abs/2001CeMDA..80...39L},
      adsnote = {Provided by the SAO/NASA Astrophysics Data System}
}
"""


    if sim.simulationarchive_auto_interval!=0 or sim.simulationarchive_auto_walltime!=0 or sim.simulationarchive_auto_step!=0:
        txt += """The SimulationArchive format was used to store fully reproducible simulation data \citep{reboundsa}. """
        bib += """@ARTICLE{reboundsa,
       author = {{Rein}, Hanno and {Tamayo}, Daniel},
        title = "{A new paradigm for reproducing and analyzing N-body simulations of planetary systems}",
      journal = {\mnras},
     keywords = {methods: numerical, gravitation, planets and satellites: dynamical evolution and stability, Astrophysics - Earth and Planetary Astrophysics, Astrophysics - Instrumentation and Methods for Astrophysics},
         year = 2017,
        month = may,
       volume = {467},
       number = {2},
        pages = {2377-2383},
          doi = {10.1093/mnras/stx232},
archivePrefix = {arXiv},
       eprint = {1701.07423},
 primaryClass = {astro-ph.EP},
       adsurl = {https://ui.adsabs.harvard.edu/abs/2017MNRAS.467.2377R},
      adsnote = {Provided by the SAO/NASA Astrophysics Data System}
}
"""
    if sim.N_var>0:
        txt += """Variational equations were used to calculate trajectories of nearby orbits \citep{reboundvar}. """
        bib += """@ARTICLE{reboundvar,
       author = {{Rein}, Hanno and {Tamayo}, Daniel},
        title = "{Second-order variational equations for N-body simulations}",
      journal = {\mnras},
     keywords = {gravitation, methods: numerical, planets and satellites: dynamical evolution and stability, Astrophysics - Earth and Planetary Astrophysics, Astrophysics - Instrumentation and Methods for Astrophysics, Mathematics - Classical Analysis and ODEs, Mathematics - Dynamical Systems},
         year = 2016,
        month = jul,
       volume = {459},
       number = {3},
        pages = {2275-2285},
          doi = {10.1093/mnras/stw644},
archivePrefix = {arXiv},
       eprint = {1603.03424},
 primaryClass = {astro-ph.EP},
       adsurl = {https://ui.adsabs.harvard.edu/abs/2016MNRAS.459.2275R},
      adsnote = {Provided by the SAO/NASA Astrophysics Data System}
}
"""




    return txt +"\n\n\n"+bib



import rebound
import unittest
import warnings

class TestPlotting(unittest.TestCase):
    def setUp(self):
        self.sim = rebound.Simulation()
        self.sim.add(m=1)
        self.sim.add(m=1e-3,a=1,e=0.1,omega=0.1,M=0.1,inc=0.1,Omega=0.1)
        self.sim.add(m=1e-3,a=-2,e=1.1,omega=0.1,M=0.1,inc=0.1,Omega=0.1)
    
    def tearDown(self):
        self.sim = None
    
    def test_orbitplot(self):
        with warnings.catch_warnings(record=True) as w: 
            warnings.simplefilter("always")
            import matplotlib; matplotlib.use("pdf")
            import numpy as np
            t = np.array(1.)
            plot, ax_main = rebound.OrbitPlot(self.sim,periastron=True)
            self.assertIsInstance(plot,matplotlib.figure.Figure)
            plot, ax_main = rebound.OrbitPlot(self.sim,periastron=True,color=True,unitlabel="AU")
            self.assertIsInstance(plot,matplotlib.figure.Figure)
    
    def test_orbitplot_slices(self):
        with warnings.catch_warnings(record=True) as w: 
            warnings.simplefilter("always")
            import matplotlib; matplotlib.use("pdf")
            import numpy as np
            t = np.array(1.)
            plot, ax_main, ax_top, ax_right  = rebound.OrbitPlot(self.sim,periastron=True,slices=True)
            self.assertIsInstance(plot,matplotlib.figure.Figure)
            plot,ax_main, ax_top, ax_right = rebound.OrbitPlot(self.sim,periastron=True,color=True,orbit_type="solid",unitlabel="AU",slices=0.4,xlim=[-1.,1])
            self.assertIsInstance(plot,matplotlib.figure.Figure)
   

if __name__ == "__main__":
    unittest.main()



import rebound
import unittest
import os

class TestUnits(unittest.TestCase):
    def setUp(self):
        self.sim = rebound.Simulation()
    
    def tearDown(self):
        self.sim = None
    
    def test_units_no_particle(self):
        default_units = {'length':None, 'mass':None, 'time':None}
        self.assertEqual(self.sim.units, default_units)
        new_units = {'length':'au', 'mass':'msun', 'time': 'yr2pi'}
        self.sim.units = ("au", "msun", "yr2pi")
        self.assertEqual(self.sim.units, new_units)
        
        self.sim.add(m=1.)
        with self.assertRaises(AttributeError):
            self.sim.units = ("au", "kg", "yr2pi")
    
    def test_wrong_units(self):
        with self.assertRaises(Exception):
            self.sim.units = ("au", "yr2pi")
        with self.assertRaises(Exception):
            self.sim.units = ("au", "bogusunit", "yr2pi")
    
    def test_units_with_particle(self):
        self.sim.units = ("au", "msun", "yr2pi")
        self.sim.add(m=1.)
        self.sim.add(m=1., x=1., vx=1.)
        self.sim.convert_particle_units("au", "kg", "yr2pi")
        self.assertAlmostEqual(self.sim.particles[0].m,1.9884754159665356e+30, delta=1e-15)
        self.sim.convert_particle_units("m", "kg", "yr2pi")
        self.assertAlmostEqual(self.sim.particles[1].x,149597870700.0, delta=1e-15)
        self.sim.convert_particle_units("m", "kg", "s")
        self.assertAlmostEqual(self.sim.particles[1].vx,29784.691834383168, delta=1e-15)
    
    def test_units_restore(self):
        units = ["au", "msun", "yr2pi"]
        self.sim.units = units 
        self.sim.save("test.bin")
        sim2 = rebound.Simulation("test.bin")
        for i in ["length","time","mass"]:
            self.assertEqual(sim2.units[i], self.sim.units[i])
            self.assertIsNotNone(sim2.units[i])

if __name__ == "__main__":
    unittest.main()



import rebound
import unittest
import math
import numpy as np

class TestLineTreeCollisions(unittest.TestCase):
    
    def test_linetree_find(self):
        # Should find the collision
        sim = rebound.Simulation()
        sim.integrator = "leapfrog"
        sim.collision  = "linetree"
        sim.configure_box(100)
        sim.dt = 10
        sim.add(r=1,x=0)
        sim.add(r=1,x=3,vx=-1)
        with self.assertRaises(rebound.Collision) as context:
            sim.integrate(10)
    def test_linetree_find_overlap1(self):
        # Should find the collision which is already overlapping at t=0
        sim = rebound.Simulation()
        sim.integrator = "leapfrog"
        sim.collision  = "linetree"
        sim.configure_box(100)
        sim.dt = 10
        sim.add(r=1,x=0)
        sim.add(r=1,x=1,vx=-1)
        with self.assertRaises(rebound.Collision) as context:
            sim.integrate(10)
    def test_linetree_find_overlap2(self):
        # Should find the collision which is not overlapping at t=0, only at end
        sim = rebound.Simulation()
        sim.integrator = "leapfrog"
        sim.collision  = "linetree"
        sim.configure_box(100)
        sim.dt = 10
        sim.add(r=1,x=0)
        sim.add(r=1,x=11,vx=-1)
        with self.assertRaises(rebound.Collision) as context:
            sim.integrate(10)
    def test_linetree_miss(self):
        sim = rebound.Simulation()
        sim.integrator = "leapfrog"
        sim.collision  = "linetree"
        sim.configure_box(100)
        sim.dt = 10
        sim.add(r=1,x=0)
        sim.add(r=1,x=2.1,vx=1)
        sim.integrate(10)



class TestLineCollisions(unittest.TestCase):
    
    def test_direct_miss(self):
        # Should miss the collision
        sim = rebound.Simulation()
        sim.integrator = "leapfrog"
        sim.collision  = "direct"
        sim.dt = 10
        sim.add(r=1,x=0)
        sim.add(r=1,x=3,vx=-1)
        sim.init_megno()
        sim.particles[-1].x = np.nan # test that collisions with variational particles not being checked
        sim.integrate(10)
    def test_line_find(self):
        # Should find the collision
        sim = rebound.Simulation()
        sim.integrator = "leapfrog"
        sim.collision  = "line"
        sim.dt = 10
        sim.add(r=1,x=0)
        sim.add(r=1,x=3,vx=-1)
        sim.init_megno()
        sim.particles[-1].x = np.nan
        with self.assertRaises(rebound.Collision) as context:
            sim.integrate(10)
    def test_line_find_overlap1(self):
        # Should find the collision already overlapping at t=0
        sim = rebound.Simulation()
        sim.integrator = "leapfrog"
        sim.collision  = "line"
        sim.dt = 10
        sim.add(r=1,x=0)
        sim.add(r=1,x=1,vx=-1)
        sim.init_megno()
        sim.particles[-1].x = np.nan
        with self.assertRaises(rebound.Collision) as context:
            sim.integrate(10)
    def test_line_find_overlap2(self):
        # Should find the collision which is not overlapping at t=0, only at end
        sim = rebound.Simulation()
        sim.integrator = "leapfrog"
        sim.collision  = "line"
        sim.dt = 10
        sim.add(r=1,x=0)
        sim.add(r=1,x=11,vx=-1)
        sim.init_megno()
        sim.particles[-1].x = np.nan
        with self.assertRaises(rebound.Collision) as context:
            sim.integrate(10)
    def test_line_miss(self):
        sim = rebound.Simulation()
        sim.integrator = "leapfrog"
        sim.collision  = "line"
        sim.configure_box(100)
        sim.dt = 10
        sim.add(r=1,x=0)
        sim.add(r=1,x=2.1,vx=1)
        sim.integrate(10)


class TestCollisions(unittest.TestCase):
    
    def test_tree_remove_both(self):
        sim = rebound.Simulation()
        boxsize = 50000.           
        sim.configure_box(boxsize)
        sim.integrator = "leapfrog"
        sim.boundary   = "open"
        sim.gravity    = "tree"
        sim.collision  = "tree"
        sim.collision_resolve = "hardsphere"
        def cor_remove_both(r, c):
            r.contents.collisions_Nlog += 1
            return 3
        sim.collision_resolve = cor_remove_both
        
        while sim.N< 10:
            sim.add(m=1., r=100., x=np.random.uniform(-20,20),
                                y=np.random.uniform(-20,20),
                                z=np.random.uniform(-20,20))
        sim.dt = 0.001
        with self.assertRaises(rebound.NoParticles):
            sim.integrate(1000.)
        self.assertEqual(sim.collisions_Nlog,5)
    
    def test_direct_remove_both(self):
        sim = rebound.Simulation()
        boxsize = 50000.           
        sim.configure_box(boxsize)
        sim.integrator = "leapfrog"
        sim.boundary   = "open"
        sim.collision  = "direct"
        def cor_remove_both(r, c):
            r.contents.collisions_Nlog += 1
            return 3
        sim.collision_resolve = cor_remove_both
        
        while sim.N< 10:
            sim.add(m=1., r=100., x=np.random.uniform(-20,20),
                                y=np.random.uniform(-20,20),
                                z=np.random.uniform(-20,20))
        sim.dt = 0.001
        with self.assertRaises(rebound.NoParticles):
            sim.integrate(1000.)
        self.assertEqual(sim.collisions_Nlog,5)

    def test_tree_remove_one(self):
        sim = rebound.Simulation()
        boxsize = 50000.           
        sim.configure_box(boxsize)
        sim.integrator = "leapfrog"
        sim.boundary   = "open"
        sim.gravity    = "tree"
        sim.collision  = "tree"
        def cor_remove_both(r, c):
            r.contents.collisions_Nlog += 1
            return np.random.randint(1,3)
        sim.collision_resolve = cor_remove_both
        
        while sim.N< 50:
            sim.add(m=1., r=100., x=np.random.uniform(-2,2),
                                y=np.random.uniform(-2,2),
                                z=np.random.uniform(-2,2))
        sim.dt = 0.001
        sim.integrate(sim.dt)
        sim.integrate(2.*sim.dt)
        self.assertLess(sim.N,25)
    
    def test_direct_remove_one(self):
        sim = rebound.Simulation()
        boxsize = 50000.           
        sim.configure_box(boxsize)
        sim.integrator = "leapfrog"
        sim.boundary   = "open"
        sim.collision  = "direct"
        def cor_remove_both(r, c):
            r.contents.collisions_Nlog += 1
            return np.random.randint(1,3)
        sim.collision_resolve = cor_remove_both
        
        while sim.N< 50:
            sim.add(m=1., r=100., x=np.random.uniform(-2,2),
                                y=np.random.uniform(-2,2),
                                z=np.random.uniform(-2,2))
        sim.dt = 0.001
        sim.integrate(sim.dt)
        sim.integrate(2.*sim.dt)
        self.assertLess(sim.N,25)

if __name__ == "__main__":
    unittest.main()



import rebound
import numpy as np
import unittest
import ctypes

class TestOrbitalElements(unittest.TestCase):
    def test_add_errors(self):
        sim = rebound.Simulation()
        with self.assertRaises(ValueError):
            sim.add(a=1)
        sim.add(m=1.)
        with self.assertRaises(ValueError):
            sim.add(a=1, e=1)
        with self.assertRaises(ValueError):
            sim.add(a=1, e=-2)
        with self.assertRaises(ValueError):
            sim.add(a=1, e=1.2)
        with self.assertRaises(ValueError):
            sim.add(a=-1, e=.2)
        with self.assertRaises(ValueError):
            sim.add(a=-1, e=2.2, f=3.)
    
    def test_calculate_orbit_errors(self):
        sim = rebound.Simulation()
        sim.add()
        sim.add(x=1.)
        with self.assertRaises(ValueError):
            a = sim.particles[1].a
        
        sim = rebound.Simulation()
        sim.add(m=1.)
        sim.add(m=1.e-3)
        with self.assertRaises(ValueError):
            a = sim.particles[1].a

    def test_inclined_eccentric(self):
        sim = rebound.Simulation()
        d = 1.e-12 # abs error tolerance
        sim.add(m=1.2354)
        sim.add(m=1.e-4,a=1.24,e=0.123,inc=0.14,omega=0.12,Omega=0.64,l=0.632)
        sim.add(m=1.e-4,a=2.24,e=0.123,inc=0.14,omega=0.12,Omega=0.64,f=0.632)
        sim.add(m=1.e-4,a=3.24,e=0.123,inc=0.14,omega=0.12,Omega=0.64,theta=0.632)
        sim.add(m=1.e-4,a=4.24,e=0.123,inc=0.14,omega=0.12,Omega=0.64,M=0.632)
        sim.add(m=1.e-4,a=5.24,e=0.123,inc=0.14,omega=0.12,Omega=0.64,T=0.632)
        sim.add(m=1.e-4,a=6.24,e=0.123,inc=0.14,pomega=0.12,Omega=0.64,l=0.632)
        sim.add(m=1.e-4,a=7.24,e=0.123,inc=0.14,pomega=0.12,Omega=0.64,f=0.632)
        sim.add(m=1.e-4,a=8.24,e=0.123,inc=0.14,pomega=0.12,Omega=0.64,theta=0.632)
        sim.add(m=1.e-4,a=9.24,e=0.123,inc=0.14,pomega=0.12,Omega=0.64,M=0.632)
        sim.add(m=1.e-4,a=10.24,e=0.123,inc=0.14,pomega=0.12,Omega=0.64,T=0.632)

        def test_p(p):
            self.assertAlmostEqual(p.e, 0.123, delta=d)
            self.assertAlmostEqual(p.inc, 0.14, delta=d)
            self.assertAlmostEqual(p.Omega, 0.64, delta=d)

        ps = sim.particles
        for p in ps[1:]: 
            test_p(p)
           
        self.assertAlmostEqual(ps[1].a, 1.24, delta=d)
        self.assertAlmostEqual(ps[1].omega, 0.12, delta=d)
        self.assertAlmostEqual(ps[1].l, 0.632, delta=d)
        self.assertAlmostEqual(ps[2].a, 2.24, delta=d)
        self.assertAlmostEqual(ps[2].omega, 0.12, delta=d)
        self.assertAlmostEqual(ps[2].f, 0.632, delta=d)
        self.assertAlmostEqual(ps[3].a, 3.24, delta=d)
        self.assertAlmostEqual(ps[3].omega, 0.12, delta=d)
        self.assertAlmostEqual(ps[3].theta, 0.632, delta=d)
        self.assertAlmostEqual(ps[4].a, 4.24, delta=d)
        self.assertAlmostEqual(ps[4].omega, 0.12, delta=d)
        self.assertAlmostEqual(ps[4].M, 0.632, delta=d)
        self.assertAlmostEqual(ps[5].a, 5.24, delta=d)
        self.assertAlmostEqual(ps[5].omega, 0.12, delta=d)
        self.assertAlmostEqual(ps[5].T, 0.632, delta=d)
        self.assertAlmostEqual(ps[6].a, 6.24, delta=d)
        self.assertAlmostEqual(ps[6].pomega, 0.12, delta=d)
        self.assertAlmostEqual(ps[6].l, 0.632, delta=d)
        self.assertAlmostEqual(ps[7].a, 7.24, delta=d)
        self.assertAlmostEqual(ps[7].pomega, 0.12, delta=d)
        self.assertAlmostEqual(ps[7].f, 0.632, delta=d)
        self.assertAlmostEqual(ps[8].a, 8.24, delta=d)
        self.assertAlmostEqual(ps[8].pomega, 0.12, delta=d)
        self.assertAlmostEqual(ps[8].theta, 0.632, delta=d)
        self.assertAlmostEqual(ps[9].a, 9.24, delta=d)
        self.assertAlmostEqual(ps[9].pomega, 0.12, delta=d)
        self.assertAlmostEqual(ps[9].M, 0.632, delta=d)
        self.assertAlmostEqual(ps[10].a, 10.24, delta=d)
        self.assertAlmostEqual(ps[10].pomega, 0.12, delta=d)
        self.assertAlmostEqual(ps[10].T, 0.632, delta=d)
    
    def test_planar_eccentric(self):
        sim = rebound.Simulation()
        d = 1.e-12 # abs error tolerance
        sim.add(m=1.2354)
        sim.add(m=1.e-4,a=1.00,e=0.123) # dummy, make sure jacobi com is calculated right for higher indices
        sim.add(m=1.e-4,a=1.24,e=0.123,pomega=0.12,l=0.632)
        sim.add(m=1.e-4,a=2.24,e=0.123,pomega=0.12,f=0.632)
        sim.add(m=1.e-4,a=3.24,e=0.123,pomega=0.12,theta=0.632)
        sim.add(m=1.e-4,a=4.24,e=0.123,pomega=0.12,M=0.632)
        sim.add(m=1.e-4,a=5.24,e=0.123,pomega=0.12,T=0.632)
        sim.add(m=1.e-4,a=6.24,e=0.123,Omega=1.25,pomega=0.12,l=0.632)
        sim.add(m=1.e-4,a=7.24,e=0.123,Omega=1.25,pomega=0.12,f=0.632)
        sim.add(m=1.e-4,a=8.24,e=0.123,Omega=1.25,pomega=0.12,theta=0.632)
        sim.add(m=1.e-4,a=9.24,e=0.123,Omega=1.25,pomega=0.12,M=0.632)
        sim.add(m=1.e-4,a=10.24,e=0.123,Omega=1.25,pomega=0.12,T=6.32)

        def test_p(p):
            self.assertAlmostEqual(p.e, 0.123, delta=d)

        ps = sim.particles
        for p in ps[1:]: 
            test_p(p)
           
        self.assertAlmostEqual(ps[2].a, 1.24, delta=d)
        self.assertAlmostEqual(ps[2].pomega, 0.12, delta=d)
        self.assertAlmostEqual(ps[2].l, 0.632, delta=d)
        self.assertAlmostEqual(ps[3].a, 2.24, delta=d)
        self.assertAlmostEqual(ps[3].pomega, 0.12, delta=d)
        self.assertAlmostEqual(ps[3].f, 0.632, delta=d)
        self.assertAlmostEqual(ps[4].a, 3.24, delta=d)
        self.assertAlmostEqual(ps[4].pomega, 0.12, delta=d)
        self.assertAlmostEqual(ps[4].theta, 0.632, delta=d)
        self.assertAlmostEqual(ps[5].a, 4.24, delta=d)
        self.assertAlmostEqual(ps[5].pomega, 0.12, delta=d)
        self.assertAlmostEqual(ps[5].M, 0.632, delta=d)
        self.assertAlmostEqual(ps[6].a, 5.24, delta=d)
        self.assertAlmostEqual(ps[6].pomega, 0.12, delta=d)
        self.assertAlmostEqual(ps[6].T, 0.632, delta=d)
        self.assertAlmostEqual(ps[7].a, 6.24, delta=d)
        self.assertAlmostEqual(ps[7].pomega, 0.12, delta=d)
        self.assertAlmostEqual(ps[7].l, 0.632, delta=d)
        self.assertAlmostEqual(ps[8].a, 7.24, delta=d)
        self.assertAlmostEqual(ps[8].pomega, 0.12, delta=d)
        self.assertAlmostEqual(ps[8].f, 0.632, delta=d)
        self.assertAlmostEqual(ps[9].a, 8.24, delta=d)
        self.assertAlmostEqual(ps[9].pomega, 0.12, delta=d)
        self.assertAlmostEqual(ps[9].theta, 0.632, delta=d)
        self.assertAlmostEqual(ps[10].a, 9.24, delta=d)
        self.assertAlmostEqual(ps[10].pomega, 0.12, delta=d)
        self.assertAlmostEqual(ps[10].M, 0.632, delta=d)
        self.assertAlmostEqual(ps[11].a, 10.24, delta=d)
        self.assertAlmostEqual(ps[11].pomega, 0.12, delta=d)
        self.assertAlmostEqual(ps[11].T, 6.32, delta=d)
    
    def test_inclined_circular(self):
        sim = rebound.Simulation()
        d = 1.e-12 # abs error tolerance
        sim.add(m=1.2354)
        sim.add(m=1.e-4,a=1.00,inc=0.14,Omega=0.64) # dummy, make sure jacobi com is calculated right for higher indices
        sim.add(m=1.e-4,a=1.24,inc=0.14,Omega=0.64,l=0.632)
        sim.add(m=1.e-4,a=2.24,inc=0.14,Omega=0.64,theta=0.632)

        def test_p(p):
            self.assertAlmostEqual(p.e, 0., delta=d)
            self.assertAlmostEqual(p.inc, 0.14, delta=d)
            self.assertAlmostEqual(p.Omega, 0.64, delta=d)

        ps = sim.particles
        for p in ps[1:]: 
            test_p(p)
           
        self.assertAlmostEqual(ps[2].a, 1.24, delta=d)
        self.assertAlmostEqual(ps[2].l, 0.632, delta=d)
        self.assertAlmostEqual(ps[3].a, 2.24, delta=d)
        self.assertAlmostEqual(ps[3].theta, 0.632, delta=d)
    
    def test_planar_circular(self):
        sim = rebound.Simulation()
        d = 1.e-12 # abs error tolerance
        sim.add(m=1.2354)
        sim.add(m=1.e-4,a=1.00) # dummy, make sure jacobi com is calculated right for higher indices
        sim.add(m=1.e-4,a=1.24,l=0.632)
        sim.add(m=1.e-4,a=2.24,theta=0.632)

        def test_p(p):
            self.assertAlmostEqual(p.e, 0., delta=d)
            self.assertAlmostEqual(p.inc, 0., delta=d)

        ps = sim.particles
        for p in ps[1:]: 
            test_p(p)
           
        self.assertAlmostEqual(ps[2].a, 1.24, delta=d)
        self.assertAlmostEqual(ps[2].l, 0.632, delta=d)
        self.assertAlmostEqual(ps[3].a, 2.24, delta=d)
        self.assertAlmostEqual(ps[3].theta, 0.632, delta=d)

    def test_retrograde_inclined_eccentric(self):
        sim = rebound.Simulation()
        d = 1.e-12 # abs error tolerance
        sim.add(m=1.2354)
        sim.add(m=1.e-4,a=1.24,e=0.123,inc=2.14,omega=0.12,Omega=0.64,l=0.632)
        sim.add(m=1.e-4,a=2.24,e=0.123,inc=2.14,omega=0.12,Omega=0.64,f=0.632)
        sim.add(m=1.e-4,a=3.24,e=0.123,inc=2.14,omega=0.12,Omega=0.64,theta=0.632)
        sim.add(m=1.e-4,a=4.24,e=0.123,inc=2.14,omega=0.12,Omega=0.64,M=0.632)
        sim.add(m=1.e-4,a=5.24,e=0.123,inc=2.14,omega=0.12,Omega=0.64,T=0.632)
        sim.add(m=1.e-4,a=6.24,e=0.123,inc=2.14,pomega=0.12,Omega=0.64,l=0.632)
        sim.add(m=1.e-4,a=7.24,e=0.123,inc=2.14,pomega=0.12,Omega=0.64,f=0.632)
        sim.add(m=1.e-4,a=8.24,e=0.123,inc=2.14,pomega=0.12,Omega=0.64,theta=0.632)
        sim.add(m=1.e-4,a=9.24,e=0.123,inc=2.14,pomega=0.12,Omega=0.64,M=0.632)
        sim.add(m=1.e-4,a=10.24,e=0.123,inc=2.14,pomega=0.12,Omega=0.64,T=0.632)

        def test_p(p):
            self.assertAlmostEqual(p.e, 0.123, delta=d)
            self.assertAlmostEqual(p.inc, 2.14, delta=d)
            self.assertAlmostEqual(p.Omega, 0.64, delta=d)

        ps = sim.particles
        for p in ps[1:]: 
            test_p(p)
           
        self.assertAlmostEqual(ps[1].a, 1.24, delta=d)
        self.assertAlmostEqual(ps[1].omega, 0.12, delta=d)
        self.assertAlmostEqual(ps[1].l, 0.632, delta=d)
        self.assertAlmostEqual(ps[2].a, 2.24, delta=d)
        self.assertAlmostEqual(ps[2].omega, 0.12, delta=d)
        self.assertAlmostEqual(ps[2].f, 0.632, delta=d)
        self.assertAlmostEqual(ps[3].a, 3.24, delta=d)
        self.assertAlmostEqual(ps[3].omega, 0.12, delta=d)
        self.assertAlmostEqual(ps[3].theta, 0.632, delta=d)
        self.assertAlmostEqual(ps[4].a, 4.24, delta=d)
        self.assertAlmostEqual(ps[4].omega, 0.12, delta=d)
        self.assertAlmostEqual(ps[4].M, 0.632, delta=d)
        self.assertAlmostEqual(ps[5].a, 5.24, delta=d)
        self.assertAlmostEqual(ps[5].omega, 0.12, delta=d)
        self.assertAlmostEqual(ps[5].T, 0.632, delta=d)
        self.assertAlmostEqual(ps[6].a, 6.24, delta=d)
        self.assertAlmostEqual(ps[6].pomega, 0.12, delta=d)
        self.assertAlmostEqual(ps[6].l, 0.632, delta=d)
        self.assertAlmostEqual(ps[7].a, 7.24, delta=d)
        self.assertAlmostEqual(ps[7].pomega, 0.12, delta=d)
        self.assertAlmostEqual(ps[7].f, 0.632, delta=d)
        self.assertAlmostEqual(ps[8].a, 8.24, delta=d)
        self.assertAlmostEqual(ps[8].pomega, 0.12, delta=d)
        self.assertAlmostEqual(ps[8].theta, 0.632, delta=d)
        self.assertAlmostEqual(ps[9].a, 9.24, delta=d)
        self.assertAlmostEqual(ps[9].pomega, 0.12, delta=d)
        self.assertAlmostEqual(ps[9].M, 0.632, delta=d)
        self.assertAlmostEqual(ps[10].a, 10.24, delta=d)
        self.assertAlmostEqual(ps[10].pomega, 0.12, delta=d)
        self.assertAlmostEqual(ps[10].T, 0.632, delta=d)
    
    def test_retrorade_planar_eccentric(self):
        sim = rebound.Simulation()
        d = 1.e-12 # abs error tolerance
        sim.add(m=1.2354)
        sim.add(m=1.e-4,a=1.00,e=0.123,inc=np.pi) # dummy, make sure jacobi com is calculated right for higher indices
        sim.add(m=1.e-4,a=1.24,e=0.123,inc=np.pi,pomega=0.12,l=0.632)
        sim.add(m=1.e-4,a=2.24,e=0.123,inc=np.pi,pomega=0.12,f=0.632)
        sim.add(m=1.e-4,a=3.24,e=0.123,inc=np.pi,pomega=0.12,theta=0.632)
        sim.add(m=1.e-4,a=4.24,e=0.123,inc=np.pi,pomega=0.12,M=0.632)
        sim.add(m=1.e-4,a=5.24,e=0.123,inc=np.pi,pomega=0.12,T=0.632)
        sim.add(m=1.e-4,a=6.24,e=0.123,inc=np.pi,Omega=1.25,pomega=0.12,l=0.632)
        sim.add(m=1.e-4,a=7.24,e=0.123,inc=np.pi,Omega=1.25,pomega=0.12,f=0.632)
        sim.add(m=1.e-4,a=8.24,e=0.123,inc=np.pi,Omega=1.25,pomega=0.12,theta=0.632)
        sim.add(m=1.e-4,a=9.24,e=0.123,inc=np.pi,Omega=1.25,pomega=0.12,M=0.632)
        sim.add(m=1.e-4,a=10.24,e=0.123,inc=np.pi,Omega=1.25,pomega=0.12,T=6.32)

        def test_p(p):
            self.assertAlmostEqual(p.e, 0.123, delta=d)
            self.assertAlmostEqual(p.inc, np.pi, delta=d)

        ps = sim.particles
        for p in ps[1:]: 
            test_p(p)
           
        self.assertAlmostEqual(ps[2].a, 1.24, delta=d)
        self.assertAlmostEqual(ps[2].pomega, 0.12, delta=d)
        self.assertAlmostEqual(ps[2].l, 0.632, delta=d)
        self.assertAlmostEqual(ps[3].a, 2.24, delta=d)
        self.assertAlmostEqual(ps[3].pomega, 0.12, delta=d)
        self.assertAlmostEqual(ps[3].f, 0.632, delta=d)
        self.assertAlmostEqual(ps[4].a, 3.24, delta=d)
        self.assertAlmostEqual(ps[4].pomega, 0.12, delta=d)
        self.assertAlmostEqual(ps[4].theta, 0.632, delta=d)
        self.assertAlmostEqual(ps[5].a, 4.24, delta=d)
        self.assertAlmostEqual(ps[5].pomega, 0.12, delta=d)
        self.assertAlmostEqual(ps[5].M, 0.632, delta=d)
        self.assertAlmostEqual(ps[6].a, 5.24, delta=d)
        self.assertAlmostEqual(ps[6].pomega, 0.12, delta=d)
        self.assertAlmostEqual(ps[6].T, 0.632, delta=d)
        self.assertAlmostEqual(ps[7].a, 6.24, delta=d)
        self.assertAlmostEqual(ps[7].pomega, 0.12, delta=d)
        self.assertAlmostEqual(ps[7].l, 0.632, delta=d)
        self.assertAlmostEqual(ps[8].a, 7.24, delta=d)
        self.assertAlmostEqual(ps[8].pomega, 0.12, delta=d)
        self.assertAlmostEqual(ps[8].f, 0.632, delta=d)
        self.assertAlmostEqual(ps[9].a, 8.24, delta=d)
        self.assertAlmostEqual(ps[9].pomega, 0.12, delta=d)
        self.assertAlmostEqual(ps[9].theta, 0.632, delta=d)
        self.assertAlmostEqual(ps[10].a, 9.24, delta=d)
        self.assertAlmostEqual(ps[10].pomega, 0.12, delta=d)
        self.assertAlmostEqual(ps[10].M, 0.632, delta=d)
        self.assertAlmostEqual(ps[11].a, 10.24, delta=d)
        self.assertAlmostEqual(ps[11].pomega, 0.12, delta=d)
        self.assertAlmostEqual(ps[11].T, 6.32, delta=d)
    
    def test_retrograde_inclined_circular(self):
        sim = rebound.Simulation()
        d = 1.e-12 # abs error tolerance
        sim.add(m=1.2354)
        sim.add(m=1.e-4,a=1.00,inc=2.14,Omega=0.64) # dummy, make sure jacobi com is calculated right for higher indices
        sim.add(m=1.e-4,a=1.24,inc=2.14,Omega=0.64,l=0.632)
        sim.add(m=1.e-4,a=2.24,inc=2.14,Omega=0.64,theta=0.632)

        def test_p(p):
            self.assertAlmostEqual(p.e, 0., delta=d)
            self.assertAlmostEqual(p.inc, 2.14, delta=d)
            self.assertAlmostEqual(p.Omega, 0.64, delta=d)

        ps = sim.particles
        for p in ps[1:]: 
            test_p(p)
           
        self.assertAlmostEqual(ps[2].a, 1.24, delta=d)
        self.assertAlmostEqual(ps[2].l, 0.632, delta=d)
        self.assertAlmostEqual(ps[3].a, 2.24, delta=d)
        self.assertAlmostEqual(ps[3].theta, 0.632, delta=d)
    
    def test_retrograde_planar_circular(self):
        sim = rebound.Simulation()
        d = 1.e-12 # abs error tolerance
        sim.add(m=1.2354)
        sim.add(m=1.e-4,inc=np.pi,a=1.00) # dummy, make sure jacobi com is calculated right for higher indices
        sim.add(m=1.e-4,inc=np.pi,a=1.24,l=0.632)
        sim.add(m=1.e-4,inc=np.pi,a=2.24,theta=0.632)

        def test_p(p):
            self.assertAlmostEqual(p.e, 0., delta=d)
            self.assertAlmostEqual(p.inc, np.pi, delta=d)

        ps = sim.particles
        for p in ps[1:]: 
            test_p(p)
           
        self.assertAlmostEqual(ps[2].a, 1.24, delta=d)
        self.assertAlmostEqual(ps[2].l, 0.632, delta=d)
        self.assertAlmostEqual(ps[2].inc, np.pi, delta=d)
        self.assertAlmostEqual(ps[3].a, 2.24, delta=d)
        self.assertAlmostEqual(ps[3].theta, 0.632, delta=d)
        self.assertAlmostEqual(ps[3].inc, np.pi, delta=d)

if __name__ == "__main__":
    unittest.main()



import rebound
import unittest
import math
import numpy as np

class TestGravity(unittest.TestCase):
    
    def test_testparticle_0(self):
        sim = rebound.Simulation()
        sim.testparticle_type = 0;
        sim.add(m=1.)
        sim.add(m=1e-3, a=1.)
        sim.N_active = 1
        sim.integrate(10.)
        x0 = sim.particles[0].x
        self.assertEqual(x0, 0.)

    def test_testparticle_1(self):
        sim = rebound.Simulation()
        sim.testparticle_type = 1;
        sim.add(m=1.)
        sim.add(m=1e-3, a=1.)
        sim.N_active = 1
        sim.integrate(10.)
        x0 = sim.particles[0].x
        self.assertNotEqual(x0, 0.)
    
    def test_testparticle_comp_0(self):
        sim = rebound.Simulation()
        sim.gravity = "compensated"
        sim.testparticle_type = 0;
        sim.add(m=1.)
        sim.add(m=1e-3, a=1.)
        sim.N_active = 1
        sim.integrate(10.)
        x0 = sim.particles[0].x
        self.assertEqual(x0, 0.)

    def test_testparticle_comp_1(self):
        sim = rebound.Simulation()
        sim.gravity = "compensated"
        sim.testparticle_type = 1;
        sim.add(m=1.)
        sim.add(m=1e-3, a=1.)
        sim.N_active = 1
        sim.integrate(10.)
        x0 = sim.particles[0].x
        self.assertNotEqual(x0, 0.)




    def test_testparticle_whfast_comp_0(self):
        sim = rebound.Simulation()
        sim.gravity = "compensated"
        sim.integrator = "whfast"
        sim.testparticle_type = 0;
        sim.add(m=1.)
        sim.add(m=1e-3, a=1.)
        sim.N_active = 1
        sim.integrate(10.)
        x0 = sim.particles[0].x
        # TODO
        # Currently fails. WHFAST evolves COM, but should only include star
        #self.assertEqual(x0, 0.)

    def test_testparticle_whfast_comp_1(self):
        sim = rebound.Simulation()
        sim.gravity = "compensated"
        sim.integrator = "whfast"
        sim.dt = 1e-4
        sim.testparticle_type = 1;
        sim.add(m=1.)
        sim.add(m=1e-3, a=1.)
        sim.add(m=1e-3, a=1.4)
        sim.N_active = 2
        sim.integrate(10.)
        x0 = sim.particles[0].x
        x1 = sim.particles[1].x
        self.assertNotEqual(x0, 0.)

        sim = rebound.Simulation()
        sim.gravity = "compensated"
        sim.testparticle_type = 1;
        sim.add(m=1.)
        sim.add(m=1e-3, a=1.)
        sim.add(m=1e-3, a=1.4)
        sim.N_active = 2
        sim.integrate(10.)
        x1ias = sim.particles[1].x
        self.assertAlmostEqual(x1ias, x1,delta=1e-9)


if __name__ == "__main__":
    unittest.main()
